class
	PD_BASES

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			make_shared,
			make_initialized,
			structure_size,
			offset_of,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			internal_c_pointer := internal_c_pointer.memory_alloc (structure_size)
			is_allocated_memory := True
		end

	make_shared (a_c_pointer: POINTER)
		do
			internal_c_pointer := a_c_pointer
		end

	make_initialized (a_data: TUPLE [
				a_chars_per_limb: INTEGER_32
				a_chars_per_bit_exactly: REAL_64
				a_big_base: NATURAL_32
				a_big_base_inverted: NATURAL_32
			])
		require else
			a_data /= Void
		do
			make
			Current.chars_per_limb := a_data.a_chars_per_limb
			Current.chars_per_bit_exactly := a_data.a_chars_per_bit_exactly
			Current.big_base := a_data.a_big_base
			Current.big_base_inverted := a_data.a_big_base_inverted
		end

feature {ANY} -- fields

	chars_per_limb: INTEGER_32 assign set_chars_per_limb
		do
			Result := c__chars_per_limb (internal_c_pointer)
		end

	chars_per_bit_exactly: REAL_64 assign set_chars_per_bit_exactly
		do
			Result := c__chars_per_bit_exactly (internal_c_pointer)
		end

	big_base: NATURAL_32 assign set_big_base
		do
			Result := c__big_base (internal_c_pointer)
		end

	big_base_inverted: NATURAL_32 assign set_big_base_inverted
		do
			Result := c__big_base_inverted (internal_c_pointer)
		end

feature {ANY} -- field setters

	set_chars_per_limb (a_chars_per_limb: INTEGER_32)
		do
			c__set_chars_per_limb (internal_c_pointer, a_chars_per_limb)
		end

	set_chars_per_bit_exactly (a_chars_per_bit_exactly: REAL_64)
		do
			c__set_chars_per_bit_exactly (internal_c_pointer, a_chars_per_bit_exactly)
		end

	set_big_base (a_big_base: NATURAL_32)
		do
			c__set_big_base (internal_c_pointer, a_big_base)
		end

	set_big_base_inverted (a_big_base_inverted: NATURAL_32)
		do
			c__set_big_base_inverted (internal_c_pointer, a_big_base_inverted)
		end

feature {ANY}

	structure_size: INTEGER_32
		external
			"C inline use %"libmpz.h%""
		alias
			"return sizeof(struct bases);"
		end

	structure_alignment: INTEGER_32
		external
			"C inline use %"libmpz.h%""
		alias
			"return __alignof__(struct bases);"
		end

	offset_of (a_field_name: STRING_8): INTEGER_32
		local
			l_c_str: C_STRING
		do
			create l_c_str.make (a_field_name)
			Result := c__offset_of (l_c_str.item)
		end

feature {ANY}

	copy (a_other: like Current)
		do
			make
			internal_c_pointer.memory_copy (a_other.c_pointer, structure_size)
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result := internal_c_pointer.memory_compare (a_other.c_pointer, structure_size)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		do
			-- dummy for CE_CLASS invariant
			create Result.make_struct (<<>>)
			Result.alignment := structure_alignment
		end

	c__chars_per_limb (a_c_pointer: POINTER): INTEGER_32
		external
			"C inline use %"libmpz.h%""
		alias
			"return ((struct bases *)$a_c_pointer)->chars_per_limb;"
		end

	c__chars_per_bit_exactly (a_c_pointer: POINTER): REAL_64
		external
			"C inline use %"libmpz.h%""
		alias
			"return ((struct bases *)$a_c_pointer)->chars_per_bit_exactly;"
		end

	c__big_base (a_c_pointer: POINTER): NATURAL_32
		external
			"C inline use %"libmpz.h%""
		alias
			"return ((struct bases *)$a_c_pointer)->big_base;"
		end

	c__big_base_inverted (a_c_pointer: POINTER): NATURAL_32
		external
			"C inline use %"libmpz.h%""
		alias
			"return ((struct bases *)$a_c_pointer)->big_base_inverted;"
		end

	c__set_chars_per_limb (a_c_pointer: POINTER; a_chars_per_limb: INTEGER_32)
		external
			"C inline use %"libmpz.h%""
		alias
			"((struct bases *)$a_c_pointer)->chars_per_limb = $a_chars_per_limb;"
		end

	c__set_chars_per_bit_exactly (a_c_pointer: POINTER; a_chars_per_bit_exactly: REAL_64)
		external
			"C inline use %"libmpz.h%""
		alias
			"((struct bases *)$a_c_pointer)->chars_per_bit_exactly = $a_chars_per_bit_exactly;"
		end

	c__set_big_base (a_c_pointer: POINTER; a_big_base: NATURAL_32)
		external
			"C inline use %"libmpz.h%""
		alias
			"((struct bases *)$a_c_pointer)->big_base = $a_big_base;"
		end

	c__set_big_base_inverted (a_c_pointer: POINTER; a_big_base_inverted: NATURAL_32)
		external
			"C inline use %"libmpz.h%""
		alias
			"((struct bases *)$a_c_pointer)->big_base_inverted = $a_big_base_inverted;"
		end

	c__offset_of (a_field_name: POINTER): INTEGER_32
		external
			"C inline use %"libmpz.h%", <string.h>, <stddef.h>, <stdio.h>"
		alias
			"[
				if(strcmp($a_field_name, "chars_per_limb") == 0) return offsetof(struct bases, chars_per_limb);
				if(strcmp($a_field_name, "chars_per_bit_exactly") == 0) return offsetof(struct bases, chars_per_bit_exactly);
				if(strcmp($a_field_name, "big_base") == 0) return offsetof(struct bases, big_base);
				if(strcmp($a_field_name, "big_base_inverted") == 0) return offsetof(struct bases, big_base_inverted);
				fprintf(stderr, "struct bases.%s not found\n", $a_field_name);
				return -1;
			]"
		end

end
