class
	PD_MPF_STRUCT

inherit
	CE_UTILITIES
		redefine
			copy,
			is_equal
		end
	CE_CLASS
		redefine
			make,
			make_shared,
			make_initialized,
			structure_size,
			offset_of,
			copy,
			is_equal
		end

create
	make,
	make_shared,
	make_initialized

feature {NONE}

	make
		do
			internal_c_pointer := internal_c_pointer.memory_alloc (structure_size)
			is_allocated_memory := True
		end

	make_shared (a_c_pointer: POINTER)
		do
			internal_c_pointer := a_c_pointer
		end

	make_initialized (a_data: TUPLE [
				a_mp_prec: INTEGER_32
				a_mp_size: INTEGER_32
				a_mp_exp: INTEGER_32
				a_mp_d: CE_POINTER [NATURAL_32]
			])
		require else
			a_data /= Void
		do
			make
			Current.mp_prec := a_data.a_mp_prec
			Current.mp_size := a_data.a_mp_size
			Current.mp_exp := a_data.a_mp_exp
			Current.mp_d := a_data.a_mp_d
		end

feature {ANY} -- fields

	mp_prec: INTEGER_32 assign set_mp_prec
		do
			Result := c__mp_prec (internal_c_pointer)
		end

	mp_size: INTEGER_32 assign set_mp_size
		do
			Result := c__mp_size (internal_c_pointer)
		end

	mp_exp: INTEGER_32 assign set_mp_exp
		do
			Result := c__mp_exp (internal_c_pointer)
		end

	mp_d: CE_POINTER [NATURAL_32] assign set_mp_d
		do
			create {CE_NATURAL_32_POINTER} Result.make_shared (c__mp_d (internal_c_pointer))
		end

feature {ANY} -- field setters

	set_mp_prec (a_mp_prec: INTEGER_32)
		do
			c__set_mp_prec (internal_c_pointer, a_mp_prec)
		end

	set_mp_size (a_mp_size: INTEGER_32)
		do
			c__set_mp_size (internal_c_pointer, a_mp_size)
		end

	set_mp_exp (a_mp_exp: INTEGER_32)
		do
			c__set_mp_exp (internal_c_pointer, a_mp_exp)
		end

	set_mp_d (a_mp_d: CE_POINTER [NATURAL_32])
		do
			c__set_mp_d (internal_c_pointer, a_mp_d.c_pointer)
		end

feature {ANY}

	structure_size: INTEGER_32
		external
			"C inline use %"libmpz.h%""
		alias
			"return sizeof(struct __anonstruct___mpf_struct_3);"
		end

	structure_alignment: INTEGER_32
		external
			"C inline use %"libmpz.h%""
		alias
			"return __alignof__(struct __anonstruct___mpf_struct_3);"
		end

	offset_of (a_field_name: STRING_8): INTEGER_32
		local
			l_c_str: C_STRING
		do
			create l_c_str.make (a_field_name)
			Result := c__offset_of (l_c_str.item)
		end

feature {ANY}

	copy (a_other: like Current)
		do
			make
			internal_c_pointer.memory_copy (a_other.c_pointer, structure_size)
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result := internal_c_pointer.memory_compare (a_other.c_pointer, structure_size)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		do
			-- dummy for CE_CLASS invariant
			create Result.make_struct (<<>>)
			Result.alignment := structure_alignment
		end

	c__mp_prec (a_c_pointer: POINTER): INTEGER_32
		external
			"C inline use %"libmpz.h%""
		alias
			"return ((struct __anonstruct___mpf_struct_3 *)$a_c_pointer)->_mp_prec;"
		end

	c__mp_size (a_c_pointer: POINTER): INTEGER_32
		external
			"C inline use %"libmpz.h%""
		alias
			"return ((struct __anonstruct___mpf_struct_3 *)$a_c_pointer)->_mp_size;"
		end

	c__mp_exp (a_c_pointer: POINTER): INTEGER_32
		external
			"C inline use %"libmpz.h%""
		alias
			"return ((struct __anonstruct___mpf_struct_3 *)$a_c_pointer)->_mp_exp;"
		end

	c__mp_d (a_c_pointer: POINTER): POINTER
		external
			"C inline use %"libmpz.h%""
		alias
			"return (EIF_POINTER)((struct __anonstruct___mpf_struct_3 *)$a_c_pointer)->_mp_d;"
		end

	c__set_mp_prec (a_c_pointer: POINTER; a_mp_prec: INTEGER_32)
		external
			"C inline use %"libmpz.h%""
		alias
			"((struct __anonstruct___mpf_struct_3 *)$a_c_pointer)->_mp_prec = $a_mp_prec;"
		end

	c__set_mp_size (a_c_pointer: POINTER; a_mp_size: INTEGER_32)
		external
			"C inline use %"libmpz.h%""
		alias
			"((struct __anonstruct___mpf_struct_3 *)$a_c_pointer)->_mp_size = $a_mp_size;"
		end

	c__set_mp_exp (a_c_pointer: POINTER; a_mp_exp: INTEGER_32)
		external
			"C inline use %"libmpz.h%""
		alias
			"((struct __anonstruct___mpf_struct_3 *)$a_c_pointer)->_mp_exp = $a_mp_exp;"
		end

	c__set_mp_d (a_c_pointer: POINTER; a_mp_d: POINTER)
		external
			"C inline use %"libmpz.h%""
		alias
			"((struct __anonstruct___mpf_struct_3 *)$a_c_pointer)->_mp_d = $a_mp_d;"
		end

	c__offset_of (a_field_name: POINTER): INTEGER_32
		external
			"C inline use %"libmpz.h%", <string.h>, <stddef.h>, <stdio.h>"
		alias
			"[
				if(strcmp($a_field_name, "mp_prec") == 0) return offsetof(struct __anonstruct___mpf_struct_3, _mp_prec);
				if(strcmp($a_field_name, "mp_size") == 0) return offsetof(struct __anonstruct___mpf_struct_3, _mp_size);
				if(strcmp($a_field_name, "mp_exp") == 0) return offsetof(struct __anonstruct___mpf_struct_3, _mp_exp);
				if(strcmp($a_field_name, "mp_d") == 0) return offsetof(struct __anonstruct___mpf_struct_3, _mp_d);
				fprintf(stderr, "struct __anonstruct___mpf_struct_3.%s not found\n", $a_field_name);
				return -1;
			]"
		end

end
